#!/usr/bin/env python
# coding: utf-8

import numpy as np
from matplotlib.patches import ConnectionPatch
import matplotlib.pyplot as plt
from matplotlib.ticker import PercentFormatter
import warnings

def omdrifthist1(datax, datay, x0, y0, r0, scale = '', nbins = 10, added_text = ''):
    """
    Creates a tracking plot form the input points, with count histograms for the
    x - y axis.

    TODO add connection to jupyter notebook using pyutils.
    Args:
        datax: the array of x-coordinates.
        datay: the array of y-coordinates.
        x0: the x coordinate for the center of the circle.
        y0: the y coordinate for the center of the circle.
        r0: the radius of the circle.
        scale: the y scale.
        added_text: text to be added.

    Output:
        1 when finished.
    """

    f, ax = plt.subplots(2, 2, figsize = (14,15), sharey=False, sharex=False)

    coordsA = "data"
    coordsB = "data"

    ax1 = ax[1,0]
    highx, binsx, _x = ax[0,0].hist(datax, color = 'forestgreen', bins = nbins, weights=np.ones(len(datax)) / len(datax), fill = True, alpha = 0.2, density = False, label='x counts')
    highy, binsy, _y = ax[1,1].hist(datay, orientation = u'horizontal', bins = nbins, color = 'forestgreen', weights=np.ones(len(datay)) / len(datay), fill = True, alpha = 0.2, density = False,label='y counts')

    minx = min(datax) - 0.5
    miny = min(datay) - 0.5
    maxx = max(datax) + 0.5
    maxy = max(datay) + 0.5

    ax[0,1].axis('off')
    ax[0,1].text(0,0, added_text, fontsize=12)
    ax[0,0].legend()
    ax[1,1].legend()
    ax[1,0].scatter(datax, datay, marker = '.', label = 'Scattered points')

    # fixing axes...
    ax[0,0].set_ylim(0, max(highy) + 0.25)
    ax[0,0].set_xlim(minx, maxx)
    ax[1,0].set_ylim(miny, maxy)
    ax[1,1].set_ylim(miny, maxy)
    ax[1,1].set_xlim(0, max(highx) + 0.25)
    ax[1,0].set_xlim(minx, maxx)

    ax[0,0].set_ylabel('Percentage out of {} pixels.'.format(len(datax)))
    ax[1,1].set_xlabel('Percentage out of {} pixels.'.format(len(datax)))
    ax[1,0].set_xlabel('x (pixels)')
    ax[1,0].set_ylabel('y (pixels)')

    #drawing arrows...
    for i in range(0, len(datay) - 1):
        xyA = (datax[i], datay[i])
        xyB = (datax[i + 1], datay[i + 1])
        con = ConnectionPatch(xyA, xyB, coordsA, coordsB,
                      arrowstyle = "-|>", shrinkA = 5, shrinkB = 5,
                      mutation_scale = 20, fc = "w")
        ax1.add_artist(con)
    circle_region = plt.Circle((x0, y0), r0, color = 'b', fill=False, label = 'Nominal tracking circle')

    #add circle and legend to the plotting subplot
    ax[1,0].add_patch(circle_region)
    ax[1,0].legend()

    f.tight_layout()

    ax[0,0].yaxis.set_major_formatter(PercentFormatter(1))
    ax[1,1].xaxis.set_major_formatter(PercentFormatter(1))

    if scale != '':
        try:
            plt.yscale(scale)
        except ValueError:
            warnings.warn('The given scale is not supported. Using linear scale instead.')
            plt.yscale('linear')

    f.suptitle('Tracking history vector plot.', x = 0.7, fontsize=18)
    plt.savefig('ejemplo.png')

    return 1


def test_omdrifthist1():
    datax = np.random.rand(40) * 4
    datay = np.random.rand(40) * 4
    omdrifthist1(datax, datay, 1.5,3.5, 1.3, '', 30, "extra text to be added\n and more text")


def omdrifthist2(driftdata, nframes, datax, datay, incremental_drift, scale = '', nbins = 10, added_text = ''):
    """
    Presents plots with data related to the drift for the tracking histogram:
     - the histogram with the incremental drift sizes.
     - the roll drift according to each frame.
     - the incremental drift spots coordinates.

    TODO: add the connection to jupyter notebook.

    Args:
        driftdata: the roll frames data.
        nframes: an array with the frames for the roll drift plot.
        datax: an array with the x coordinates of the incremental drift spots.
        datay: an array with the y coordinates of the incremental drift spots.

    Output:
        1 when finished.
    """

    f, ax = plt.subplots(2,2, figsize = (14,14), sharey=False, sharex=False)

    ax1 = ax[1,0]
    ax[0,1].scatter(datax, datay, marker = '.')
    ax[0,1].title.set_text('')
    ax[1,0].plot(nframes, driftdata, marker = '.')
    high, bins, _ = ax[1,1].hist(incremental_drift, color = 'forestgreen', fill = True, bins = nbins, alpha = 0.2, density = False)

    # plot titles...
    ax[0,1].title.set_text('Incremental drift spot diagram')
    ax[1,0].title.set_text('Roll drift plot')
    ax[1,1].title.set_text('Drift size histogram')

    # left empty to add text
    ax[0,0].axis('off')
    ax[0,0].text(0,0, added_text, fontsize=12)

    ax[0,1].legend()
    ax[1,1].legend()
    ax[1,0].scatter(datax, datay, marker = '.', label = 'Scattered points')

    ax[1,0].legend()

    f.tight_layout()

    if scale != '':
        try:
            plt.yscale(scale)
        except ValueError:
            warnings.warn('The given scale is not supported. Using linear scale instead.')
            plt.yscale('linear')

    return 1


def test_omdrifthist():
    datax = np.random.rand(30) * 3
    datay = np.random.rand(30) * 3
    nframes = np.linspace(1,100,100)
    driftdata = np.random.normal(2.1,0.3,len(nframes))
    incremental_drift = np.random.rand(30) *4
    omdrifthist2(driftdata, nframes, datax, datay, incremental_drift)
